import { BadRequestException, Injectable, NotFoundException } from "@nestjs/common";
import { DrizzleService } from "src/drizzle/drizzle.service";
import { ProductsService } from "src/products/products.service";
import { StocksTable } from "src/schema";
import { asc, desc, eq } from "drizzle-orm";


@Injectable()
export class StocksService {
  constructor(
    private readonly db: DrizzleService,
    private readonly products: ProductsService
  ) {}

  private async getProductStock(productId: string) {
    return this.db.use()
      .select()
      .from(StocksTable)
      .where(eq(StocksTable.productId, productId))
      .execute();
  }

  async createStock(productId: string, quantity: number) {
    const product = await this.products.findById(productId)
    if (!product) {
      throw new NotFoundException('Product not found');
    }
    const existingStock = await this.getProductStock(productId)
    if (existingStock.length > 0) {
      throw new BadRequestException(`Stock already exists, edit the existing one. (quantity: ${existingStock[0].quantity})`);
    }

    return this.db.use()
      .insert(StocksTable)
      .values({
        quantity: quantity,
        productId: productId,
      });
  }

  async alterStock(productId: string, newQuantity: number) {
    const product = await this.products.findById(productId)
    if (!product) {
      throw new NotFoundException('Product not found');
    }
  }

  async getCurrentStock(productId: string) {
    const product = await this.products.findById(productId)
    if (!product) {
      throw new NotFoundException('Product not found');
    }
    return product;
  }

  async getProductsRankedByStock(limit: number, _asc: boolean) {
    const order = _asc ? asc(StocksTable.quantity) : desc(StocksTable.quantity);
    return this.db.use()
      .select()
      .from(StocksTable)
      .orderBy(order)
      .limit(limit);
  }

  async getAllStocks(page: number, limit: number, _asc: boolean) {
    const order = _asc ? asc(StocksTable.quantity) : desc(StocksTable.quantity);
    return this.db.use()
      .select()
      .from(StocksTable)
      .orderBy(order)
      .offset((page - 1) * limit)
      .limit(limit);
  }

  async decrementStock(productId: string, quantityToDrop: number) {
    const product = await this.products.findById(productId)
    if (!product) {
      throw new NotFoundException('Product not found');
    }

    const existingStock = await this.getProductStock(productId)
    if (existingStock.length === 0) {
      throw new BadRequestException("Stock not exists, create an existing one.");
    }

    if (existingStock[0].quantity - quantityToDrop < 0) {
      throw new BadRequestException(`Operation not permitted, stock does not meet the quantity to do that operation. (current: ${existingStock[0].quantity}, after: ${existingStock[0].quantity - quantityToDrop})`);
    }

    return  this.db.use()
      .update(StocksTable)
      .set({
        quantity: existingStock[0].quantity - quantityToDrop,
      })
      .where(eq(StocksTable.productId, productId));
  }

  async incrementStock(productId: string, quantity: number) {
    const product = await this.products.findById(productId)
    if (!product) {
      throw new NotFoundException('Product not found');
    }
    const existingStock = await this.getProductStock(productId)
    if (existingStock.length === 0) {
      throw new BadRequestException("Stock not exists, create an existing one.");
    }
    return this.db.use()
      .update(StocksTable)
      .set({
        quantity: existingStock[0].quantity + quantity,
      })
      .where(eq(StocksTable.productId, productId));
  }
}
